'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fetch = _interopDefault(require('redux-api-call-adapter-fetch'));
var json = _interopDefault(require('redux-api-call-adapter-json'));
var dedupe = _interopDefault(require('redux-api-call-adapter-dedupe'));

var CALL_API = Symbol('CALL_API');
var REDUCER_PATH = 'api_calls';
var ACTION_FETCH_START = '@@api/FETCH_START';
var ACTION_FETCH_COMPLETE = '@@api/FETCH_COMPLETE';
var ACTION_FETCH_FAILURE = '@@api/FETCH_FAILURE';
var ACTION_UPDATE_LOCAL = '@@api/UPDATE_LOCAL';
var ACTION_RESET_LOCAL = '@@api/RESET_LOCAL';

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

















var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

var makeStartErrorAction = function makeStartErrorAction(api) {
  return function (error) {
    return {
      type: ACTION_FETCH_START,
      error: true,
      payload: _extends({
        error: error
      }, api)
    };
  };
};

var makeStartAction = function makeStartAction(api) {
  return function () {
    return {
      type: ACTION_FETCH_START,
      payload: _extends({}, api, {
        requestedAt: Date.now()
      })
    };
  };
};

var makeSuccessAction = function makeSuccessAction(api) {
  return function (payload, meta) {
    return {
      type: ACTION_FETCH_COMPLETE,
      payload: _extends({}, api, {
        json: payload,
        respondedAt: Date.now()
      }),
      meta: meta
    };
  };
};

var makeFailureAction = function makeFailureAction(api) {
  return function (payload, meta) {
    return {
      type: ACTION_FETCH_FAILURE,
      payload: _extends({}, api, {
        json: payload,
        respondedAt: Date.now()
      }),
      meta: meta
    };
  };
};

var reduceKeys = function reduceKeys(obj) {
  return function (reducer, seed) {
    return Object.keys(obj).reduce(function (acc, key) {
      return _extends({}, acc, reducer(obj, key));
    }, seed);
  };
};

var bindFunction = function bindFunction(getState) {
  return function (obj, key) {
    return defineProperty({}, key, typeof obj[key] === 'function' ? obj[key](getState()) : obj[key]);
  };
};

var applyFunctions = (function (getState) {
  return function (api) {
    return reduceKeys(api)(bindFunction(getState), {});
  };
});

var compose$1 = function compose$1() {
  for (var _len = arguments.length, adapters = Array(_len), _key = 0; _key < _len; _key++) {
    adapters[_key] = arguments[_key];
  }

  if (adapters.length === 0) {
    throw new Error('redux-api-call: composeAdatpers must take at least one adapter');
  }
  var reversed = adapters.reverse();
  var head = reversed[0];
  var tail = reversed.slice(1);

  return function (getState) {
    return tail.reduce(function (acc, current) {
      return current(acc, getState);
    }, head(function (x) {
      return x;
    }, getState));
  };
};

var defaultAdapter = compose$1(json, dedupe, fetch);
var isValid = function isValid(api) {
  return typeof api.name === 'string' && typeof api.endpoint === 'string';
};

var createAPIMiddleware = function createAPIMiddleware(adapter) {
  return function (_ref) {
    var dispatch = _ref.dispatch,
        getState = _ref.getState;

    var finalAdapter = adapter(getState);
    var resolveState = applyFunctions(getState);

    return function (next) {
      return function (action) {
        if (!action[CALL_API]) {
          next(action);
          return;
        }

        var rawRequest = action[CALL_API];
        var request = resolveState(rawRequest);

        if (!isValid(request)) {
          dispatch(makeStartErrorAction(request)());
          return;
        }

        dispatch(makeStartAction(request)());

        finalAdapter(request).then(function (response) {
          if (response) {
            dispatch(makeSuccessAction(request)(response.payload, response.meta));
          }
        }, function (failure) {
          dispatch(makeFailureAction(request)(failure.payload, failure.meta));
        });
      };
    };
  };
};

var middleware = createAPIMiddleware(defaultAdapter);

var get$1 = (function (array, defaulValue) {
  return function (state) {
    var finalValue = array.reduce(function (value, nextProp) {
      if (typeof value === 'undefined' || value === null) {
        return;
      }
      return value[nextProp];
    }, state);

    if (typeof finalValue === 'undefined') {
      return defaulValue;
    }

    return finalValue;
  };
});

var normalizeResetData = function normalizeResetData() {
  var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ['lastRequest', 'isFetching', 'isInvalidated', 'lastResponse', 'data', 'error'];

  if (typeof data === 'string') {
    return [data];
  }
  if (!Array.isArray(data)) {
    console.warn('You are using resetter wrong, the params should be string, array or undefined');
    return [];
  }
  return data;
};

var makeFetchAction = (function (apiName, apiConfigFn) {
  var selectorDescriptor = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var actionCreator = function actionCreator() {
    return defineProperty({}, CALL_API, _extends({}, apiConfigFn.apply(undefined, arguments), {
      name: apiName
    }));
  };

  var updater = function updater(data) {
    return {
      type: ACTION_UPDATE_LOCAL,
      payload: {
        name: apiName,
        data: data
      }
    };
  };

  var resetter = function resetter(data) {
    return {
      type: ACTION_RESET_LOCAL,
      payload: {
        name: apiName,
        data: normalizeResetData(data)
      }
    };
  };

  var isFetchingSelector = get$1([REDUCER_PATH, apiName, 'isFetching'], false);
  var isInvalidatedSelector = get$1([REDUCER_PATH, apiName, 'isInvalidated'], false);
  var dataSelector = get$1([REDUCER_PATH, apiName, 'data'], null);
  var headersSelector = get$1([REDUCER_PATH, apiName, 'headers'], null);
  var errorSelector = get$1([REDUCER_PATH, apiName, 'error'], null);
  var lastResponseSelector = get$1([REDUCER_PATH, apiName, 'lastResponse'], null);

  return {
    actionCreator: actionCreator,
    updater: updater,
    isFetchingSelector: isFetchingSelector,
    isInvalidatedSelector: isInvalidatedSelector,
    dataSelector: dataSelector,
    headersSelector: headersSelector,
    errorSelector: errorSelector,
    lastResponseSelector: lastResponseSelector,
    resetter: resetter
  };
});

var handleActions = (function (handlers) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    var handler = handlers[action.type];
    if (typeof handler !== 'function') {
      return state;
    }

    return handler(state, action);
  };
});

var _handleActions;

var getName = function getName(action) {
  return action.payload.name;
};
var getRequestedAt = function getRequestedAt(action) {
  return action.payload.requestedAt;
};
var getRespondedAt = function getRespondedAt(action) {
  return action.payload.respondedAt;
};
var getJSONResponse = function getJSONResponse(action) {
  return action.payload.json;
};
var getError = function getError(action) {
  return action.payload.error;
};
var getPreviousError = function getPreviousError(state, action) {
  return state[getName(action)] ? state[getName(action)].error : null;
};

var includeString = function includeString(element, array) {
  return array.indexOf(element) !== -1;
};
var resetOrKeepValue = function resetOrKeepValue(field, action, currentData) {
  return includeString(field, action.payload.data) ? undefined : currentData[field];
};

var updateWith = function updateWith(state, name, obj) {
  return _extends({}, state, defineProperty({}, name, _extends({}, state[name], obj)));
};

var reducer = handleActions((_handleActions = {}, defineProperty(_handleActions, ACTION_FETCH_START, function (state, action) {
  return updateWith(state, getName(action), {
    lastRequest: getRequestedAt(action),
    isFetching: !action.error,
    isInvalidated: true,
    error: action.error ? getError(action) : getPreviousError(state, action)
  });
}), defineProperty(_handleActions, ACTION_FETCH_COMPLETE, function (state, action) {
  return updateWith(state, getName(action), {
    isFetching: false,
    isInvalidated: false,
    lastResponse: getRespondedAt(action),
    data: getJSONResponse(action),
    error: null,
    headers: action.meta
  });
}), defineProperty(_handleActions, ACTION_FETCH_FAILURE, function (state, action) {
  return updateWith(state, getName(action), {
    isFetching: false,
    isInvalidated: true,
    error: getJSONResponse(action),
    headers: action.meta
  });
}), defineProperty(_handleActions, ACTION_UPDATE_LOCAL, function (state, action) {
  return updateWith(state, getName(action), {
    data: action.payload.data
  });
}), defineProperty(_handleActions, ACTION_RESET_LOCAL, function (state, action) {
  var name = getName(action);
  var currentData = state[name] || {};
  return updateWith(state, name, {
    lastRequest: resetOrKeepValue('lastRequest', action, currentData),
    isFetching: resetOrKeepValue('isFetching', action, currentData),
    isInvalidated: resetOrKeepValue('isInvalidated', action, currentData),
    lastResponse: resetOrKeepValue('lastResponse', action, currentData),
    data: resetOrKeepValue('data', action, currentData),
    error: resetOrKeepValue('error', action, currentData)
  });
}), _handleActions));

var reducers = defineProperty({}, REDUCER_PATH, reducer);
var ACTIONS = {
  START: ACTION_FETCH_START,
  COMPLETE: ACTION_FETCH_COMPLETE,
  FAILURE: ACTION_FETCH_FAILURE,
  UPDATE_LOCAL: ACTION_UPDATE_LOCAL
};

var defaultTransformers = [json, fetch];

exports.defaultTransformers = defaultTransformers;
exports.middleware = middleware;
exports.makeFetchAction = makeFetchAction;
exports.reducers = reducers;
exports.ACTIONS = ACTIONS;
exports.composeAdapters = compose$1;
exports.createAPIMiddleware = createAPIMiddleware;
